import requests
import time
from datetime import date
from datetime import timedelta
import datetime
import pandas as pd
import os.path
import time
import os
import csv
import json


def create_json_file(dl_root, network, version, network_provided_meta_file, meta_params):

    identifier = meta_params['identifier']
    delimiter_csv = meta_params['delimiter_meta_csv']

    # get update time of this json file
    if meta_params['network_provided_update_time'] == True:
        update_date = 'yet to implement'
    else: # use the time of download
        raw_date = network_provided_meta_file.split('META_')[-1].split('.csv')[0]
        if len(raw_date) == 4: # format YYYY
            raw_date = raw_date+'0101' # assume January 1st

        update_date = datetime.datetime.strptime(raw_date, '%Y%m%d')
        update_date = update_date.astimezone(datetime.timezone.utc).strftime('%Y-%m-%d')
        

    json_metadata = {}  
    with open(network_provided_meta_file) as file:
        csv_filedata = csv.DictReader(file, delimiter=delimiter_csv)

        for row in csv_filedata:  
            key = row[identifier]
            #update_date = datetime.datetime.utcnow().strftime('%Y-%m-%d') # check whether networks have a 'last changed' variable
            for parameter in row:
                row[parameter] = {'values': [row[parameter]], 'update_time': [update_date]} # create inner dictionary for every parameter
            json_metadata[key] = row


        # check for punctuation of lat, lon, alt, height
        for punct_variable in ['Latitude', 'Longitude', 'Altitude', 'Height']: # standard GHOST names
            if punct_variable in meta_params.keys():
                for network_spec_var_name in meta_params[punct_variable]: # loop through all possible variable names in one network, usually it should just be one name 
                    for station in json_metadata: # loop through all stations
                        if network_spec_var_name in json_metadata[station]: # clean punctuation discrepancies
                            input_val = json_metadata[station][network_spec_var_name]['values'][0] # check values
                            if input_val == '': # check if its empty string
                                output_val = None
                            elif input_val == '-':  # check if its dash
                                output_val = None
                            else:
                                output_val = input_val.replace(',', '.')
                                output_val = output_val.replace(' ', '')
                                output_val = str(float(output_val))

                            json_metadata[station][network_spec_var_name]['values'][0] = output_val
                            
                        else:
                            continue
            else:
                continue

    return json_metadata


def compare_jsons(dl_root, network, version, json_metadata_now):

    # import json basefile
    with open('{}/{}/metadata/{}/processed/{}_META.json'.format(dl_root, network, version, network), 'r', encoding='utf-8') as f:
        json_metadata = json.loads(f.read())

    k=0 # number of abolished stations
    m=0 # number of new stations
    l=0 # number of new parameters per station
    station_abolished = []
    station_new = []  
    parameter_new = []

    # compare files
    for station in json_metadata: # loop through all the old stations
        if station in json_metadata_now.keys(): # if station is in current meta data, go on
            for parameter in json_metadata[station]:
                if parameter in json_metadata_now[station]: # check if parameter exists in new meta file
                    if ((json_metadata_now[station][parameter]['values'][0] == None) or (json_metadata_now[station][parameter]['values'][0] == '')): # if new entry is NaN or empty string, disregard
                        continue
                    else: 
                        if json_metadata[station][parameter]['values'][-1] != json_metadata_now[station][parameter]['values'][0]: # compare last entry in standardised file to value in new file
                            # if different value, append the standardised metadeta file
                            print("new {} --- old {}".format(json_metadata_now[station][parameter]['values'][0], json_metadata[station][parameter]['values'][-1]))
                            json_metadata[station][parameter]['values'].append(json_metadata_now[station][parameter]['values'][0])
                            json_metadata[station][parameter]['update_time'].append(json_metadata_now[station][parameter]['update_time'][0])
                        else:
                            pass
                else: # a parameter disappeared in the new file -> do nothing
                    pass 
        else:
            k=k+1
            station_abolished.append(json_metadata[station]['station_name']['values'])


    for station in json_metadata_now: # loop through all the new stations
        if station in json_metadata.keys(): # if station is in old meta data
            pass # comparison was done before
        else: # new station appeared!
            m=m+1
            station_new.append(json_metadata_now[station]['station_name']['values'])
            json_metadata.update({station: json_metadata_now[station]})

        for parameter in json_metadata_now[station]: # loop through all the parameters
            # is there a new parameter that wasn't in the old file?
            if parameter in json_metadata[station].keys():
                pass # parameter (column) is already there
            else:
                l=l+1
                parameter_new.append(parameter)
                json_metadata[station].update({parameter: json_metadata_now[station][parameter]})
        

    # summary
    print('================', network, '================')
    print('{} stations were abolished'.format(k))
    print('Abolished stations: {}'.format(station_abolished))
    print('{} new stations '.format(m))
    print('New stations: {}'.format(station_new))
    print('{} new parameters '.format(l))
    print('New parameters per station: {}'.format(parameter_new))

    # update json basefile
    with open('{}/{}/metadata/{}/processed/{}_META.json'.format(dl_root, network, version, network), 'w', encoding='utf-8') as f:
        f.write(json.dumps(json_metadata, indent=4))